/*
|--------------------------------------------------------------------------
| Mocha Tests
|--------------------------------------------------------------------------
*/

var should = require('should');
var validate = require('../index');

describe('Validator', function () {

    describe('()', function () {

        it('should fail validation and return errors', function () {

            var form = [
                {
                    name: 'email',
                    input: 'foo@bar',
                    rules: [
                        {
                            name: 'required',
                            message: 'The email field is required.'
                        },
                        {
                            name: 'email',
                            message: 'You must provide a valid email address.'
                        }
                    ]
                },
                {
                    name: 'password',
                    input: '23ga',
                    rules: [
                        {
                            name: 'required',
                            message: 'The password field is required.'
                        },
                        {
                            name: { min: 6 },
                            message: 'The password must be at least 6 characters.'
                        }
                    ]
                }
            ];

            var passed = validate(form);
            var errors = validate.errors();

            passed.should.be.false;

            errors.email.should.have.property('rule', 'email');
            errors.email.should.have.property('message', 'You must provide a valid email address.');

            errors.password.should.have.property('rule', 'min');
            errors.password.should.have.property('message', 'The password must be at least 6 characters.');

        });

        it('should successfully validate and return an empty array', function () {

            var form = [
                {
                    name: 'email',
                    input: 'foo@bar.com',
                    rules: [
                        {
                            name: 'required',
                            message: 'The email field is required.'
                        },
                        {
                            name: 'email',
                            message: 'You must provide a valid email.'
                        }
                    ]
                }
            ];

            var passed = validate(form);
            var errors = validate.errors();

            passed.should.be.true;
            errors.email.should.be.an.object;

        });

        it('should make every rule fail', function () {

            var form = [

                {
                    name: 'a',
                    input: '',
                    rules: [
                        {
                            name: 'required',
                            message: 'it\'s required'
                        }
                    ]
                },
                {
                    name: 'b',
                    input: 'test',
                    rules: [
                        {
                            name: 'email',
                            message: 'it has to be an email'
                        }
                    ]
                },
                {
                    name: 'c',
                    input: 'www',
                    rules: [
                        {
                            name: 'url',
                            message: 'it has to be a url'
                        }
                    ]
                },
                {
                    name: 'd',
                    input: '234',
                    rules: [
                        {
                            name: 'ip',
                            message: 'it has to be an ip'
                        }
                    ]
                },
                {
                    name: 'e',
                    input: '344',
                    rules: [
                        {
                            name: 'alpha',
                            message: 'it has to be alpha'
                        }
                    ]
                },
                {
                    name: 'f',
                    input: 'abc',
                    rules: [
                        {
                            name: 'numeric',
                            message: 'it has to be numeric'
                        }
                    ]
                },
                {
                    name: 'g',
                    input: '_- /+=',
                    rules: [
                        {
                            name: 'alphanumeric',
                            message: 'it has to be alphanumeric'
                        }
                    ]
                },
                {
                    name: 'h',
                    input: 'arg8erh90gre',
                    rules: [
                        {
                            name: 'hex',
                            message: 'it has to be hex'
                        }
                    ]
                },
                {
                    name: 'i',
                    input: 'ABC',
                    rules: [
                        {
                            name: 'lowercase',
                            message: 'it has to be lowercase'
                        }
                    ]
                },
                {
                    name: 'j',
                    input: 'abc',
                    rules: [
                        {
                            name: 'uppercase',
                            message: 'it has to be uppercase'
                        }
                    ]
                },
                {
                    name: 'k',
                    input: 'abc',
                    rules: [
                        {
                            name: 'int',
                            message: 'it has to be an int'
                        }
                    ]
                },
                {
                    name: 'l',
                    input: 'abc',
                    rules: [
                        {
                            name: 'float',
                            message: 'it has to be a float'
                        }
                    ]
                },
                {
                    name: 'm',
                    input: 'abc',
                    rules: [
                        {
                            name: 'date',
                            message: 'it has to be a date'
                        }
                    ]
                },
                {
                    name: 'n',
                    input: 'abc',
                    rules: [
                        {
                            name: 'json',
                            message: 'it has to be JSON'
                        }
                    ]
                },
                {
                    name: 'o',
                    input: '8',
                    rules: [
                        {
                            name: { divisible: 3 },
                            message: 'it has to be divisible by 3'
                        }
                    ]
                },
                {
                    name: 'p',
                    input: 'a',
                    rules: [
                        {
                            name: { min: 2 },
                            message: 'it has to be at least 2'
                        }
                    ]
                },
                {
                    name: 'q',
                    input: 'abc',
                    rules: [
                        {
                            name: { max: 2 },
                            message: 'it can\'t be more than 2'
                        }
                    ]
                },
                {
                    name: 'r',
                    input: 'abc',
                    rules: [
                        {
                            name: { contains: 'z' },
                            message: 'it has to contain z'
                        }
                    ]
                }

            ];

            var pass = validate(form);
            var errors = validate.errors();

            pass.should.be.false;

            errors.a.message.should.exist;
            errors.b.message.should.exist;
            errors.c.message.should.exist;
            errors.d.message.should.exist;
            errors.e.message.should.exist;
            errors.f.message.should.exist;
            errors.g.message.should.exist;
            errors.h.message.should.exist;
            errors.i.message.should.exist;
            errors.j.message.should.exist;
            errors.k.message.should.exist;
            errors.l.message.should.exist;
            errors.m.message.should.exist;
            errors.n.message.should.exist;
            errors.o.message.should.exist;
            errors.p.message.should.exist;
            errors.q.message.should.exist;
            errors.r.message.should.exist;

        });


        it('should make every rule pass', function () {

            var form = [

                {
                    name: 'a',
                    input: 'hey',
                    rules: [
                        {
                            name: 'required',
                            message: 'it\'s required'
                        }
                    ]
                },
                {
                    name: 'b',
                    input: 'foo@bar.com',
                    rules: [
                        {
                            name: 'email',
                            message: 'it has to be an email'
                        }
                    ]
                },
                {
                    name: 'c',
                    input: 'www.pauldilyard.com',
                    rules: [
                        {
                            name: 'url',
                            message: 'it has to be a url'
                        }
                    ]
                },
                {
                    name: 'd',
                    input: '127.0.0.1',
                    rules: [
                        {
                            name: 'ip',
                            message: 'it has to be an ip'
                        }
                    ]
                },
                {
                    name: 'e',
                    input: 'abc',
                    rules: [
                        {
                            name: 'alpha',
                            message: 'it has to be alpha'
                        }
                    ]
                },
                {
                    name: 'f',
                    input: '123',
                    rules: [
                        {
                            name: 'numeric',
                            message: 'it has to be numeric'
                        }
                    ]
                },
                {
                    name: 'g',
                    input: 'abc123',
                    rules: [
                        {
                            name: 'alphanumeric',
                            message: 'it has to be alphanumeric'
                        }
                    ]
                },
                {
                    name: 'h',
                    input: 'fff',
                    rules: [
                        {
                            name: 'hex',
                            message: 'it has to be hex'
                        }
                    ]
                },
                {
                    name: 'i',
                    input: 'abc',
                    rules: [
                        {
                            name: 'lowercase',
                            message: 'it has to be lowercase'
                        }
                    ]
                },
                {
                    name: 'j',
                    input: 'ABC',
                    rules: [
                        {
                            name: 'uppercase',
                            message: 'it has to be uppercase'
                        }
                    ]
                },
                {
                    name: 'k',
                    input: '12',
                    rules: [
                        {
                            name: 'int',
                            message: 'it has to be an int'
                        }
                    ]
                },
                {
                    name: 'l',
                    input: '12.0',
                    rules: [
                        {
                            name: 'float',
                            message: 'it has to be a float'
                        }
                    ]
                },
                {
                    name: 'm',
                    input: '4/30/2014',
                    rules: [
                        {
                            name: 'date',
                            message: 'it has to be a date'
                        }
                    ]
                },
                {
                    name: 'n',
                    input: '{}',
                    rules: [
                        {
                            name: 'json',
                            message: 'it has to be JSON'
                        }
                    ]
                },
                {
                    name: 'o',
                    input: '9',
                    rules: [
                        {
                            name: { divisible: 3 },
                            message: 'it has to be divisible by 3'
                        }
                    ]
                },
                {
                    name: 'p',
                    input: 'abc',
                    rules: [
                        {
                            name: { min: 2 },
                            message: 'it has to be at least 2'
                        }
                    ]
                },
                {
                    name: 'q',
                    input: 'ab',
                    rules: [
                        {
                            name: { max: 2 },
                            message: 'it can\'t be more than 2'
                        }
                    ]
                },
                {
                    name: 'r',
                    input: 'abcz',
                    rules: [
                        {
                            name: { contains: 'z' },
                            message: 'it has to contain z'
                        }
                    ]
                }

            ];

            var pass = validate(form);
            var errors = validate.errors();

            pass.should.be.true;

            errors.a.should.be.empty;
            errors.b.should.be.empty;
            errors.c.should.be.empty;
            errors.d.should.be.empty;
            errors.e.should.be.empty;
            errors.f.should.be.empty;
            errors.g.should.be.empty;
            errors.h.should.be.empty;
            errors.i.should.be.empty;
            errors.j.should.be.empty;
            errors.k.should.be.empty;
            errors.l.should.be.empty;
            errors.m.should.be.empty;
            errors.n.should.be.empty;
            errors.o.should.be.empty;
            errors.p.should.be.empty;
            errors.q.should.be.empty;
            errors.r.should.be.empty;

        });

    });

});
